/*
 *  TOPPERS/HRP Kernel
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      High Reliable system Profile Kernel
 * 
 *  Copyright (C) 2006-2022 by Embedded and Real-Time Systems Laboratory
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 * 
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 * 
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 * 
 *  $Id: target_test.h 1143 2022-10-23 13:34:59Z ertl-hiro $
 */

/*
 *		テストプログラムのターゲット依存部（Linux用）
 */

#ifndef TOPPERS_TARGET_TEST_H
#define TOPPERS_TARGET_TEST_H

#include <signal.h>
#include <pthread.h>
#include "intno.h"

/*
 *  サンプルプログラム／テストプログラムで設定するスタックサイズ
 */
#define	STACK_SIZE		SIGSTKSZ

/*
 *  サンプルプログラム／テストプログラムで使用するCPU例外に関する定義
 */
#define CPUEXC1					EXCNO_SIGFPE
#define RAISE_CPU_EXCEPTION		pthread_kill(pthread_self(), SIGFPE)
#define PREPARE_RETURN_CPUEXC

/*
 *  サンプルプログラム／テストプログラムで使用する割込みに関する定義
 */
#define INTNO1					INTNO_SIGHUP
#define INTNO1_INTATR			TA_ENAINT|TA_EDGE
#define INTNO1_INTPRI			(-5)
#define intno1_clear()

/*
 *  サンプルプログラムのためのその他の定義
 */
#define LOOP_REF				ULONG_C(10000000)
#define MEASURE_TWICE

/*
 *  テストプログラムで使用する時間パラメータに関する定義
 */
#define TEST_TIME_CP	1000U

/*
 *  cal_svcでE_NOMEMが検出できない
 */
#define TOPPERS_NOSUPPORT_CALSVC_NOMEM

#endif /* TOPPERS_TARGET_TEST_H */
